//=============================================================================
// (c) Copyright 2005 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCADScan.c   v5.9
// Desc: Sample program that demonstrates how to perform an AD scan
// Created By KL
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif
#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
#include <mem.h>
// diamond driver includes
#include "../../../../current/dev/source/dscud.h"
#endif

// Linux and QNX
#if defined(linux) || defined(__QNXNTO__) || defined(_WRS_VXWORKS_5_X)
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"


#ifdef _WRS_VXWORKS_5_X
#include <selectLib.h>
#define main DMM48DSCADScan
#else
#include <sys/time.h>
#endif

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif


// var declarations
BYTE result;  // returned error code
DSCB dscb;    // handle used to refer to the board
DSCCB dsccb;  // structure containing board settings
DSCSAMPLE* samples;  // sample readings
DSCADSCAN dscadscan; // structure containing A/D scan settings
DSCADSETTINGS dscadsettings; // structure containing A/D conversion settings
ERRPARAMS errorParams;       // structure for returning error code and error string
DFLOAT voltage;   // actual voltage
int intBuff;      // temp variable of size int
int i;            // miscellaneous counter

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//     STEPS TO FOLLOW:
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. AD Settings Initialization
//		IV. AD Scan Initialization
//	     V. Scan and Output
//	    VI. Cleanup
//
//=============================================================================

int main( void )
{

	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the DMM48 board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf( "\nDMM-48-AT BOARD INITIALIZATION:\n" );

	printf("Enter the base address in hex: ");
  	scanf( "%hx", &dsccb.base_address );

	dsccb.int_level = 3;

	if(dscInitBoard(DSC_DMM48, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// III. AD SETTINGS INITIALIZATION
	//
	//	    Initialize the structure containing the AD conversion settings and
	//		then pass it to the driver.
	//
	//=========================================================================

	/* Prefilled exampled
	dscadsettings.range = RANGE_10;
	dscadsettings.polarity = BIPOLAR;
	dscadsettings.gain = GAIN_1;
	dscadsettings.load_cal = (BYTE)TRUE;
	dscadsettings.current_channel = 0;
	dscadsettings.scan_interval = SCAN_INTERVAL_9;
	*/

	printf("Valid Modes are 8 (+-10V), 9 (+-5V), 13 (0-5V)\n");

	memset(&dscadsettings, 0, sizeof(DSCADSETTINGS));

	printf("Enter the Mode (8, 9, 13): ");
	scanf("%d", &intBuff);
	dscadsettings.range = RANGE_10;
	dscadsettings.polarity = (intBuff >> 2) & 0x01;
	dscadsettings.gain = (intBuff & 0x03);
	printf("Enter load_cal (0 for False, 1 for True): ");
	scanf("%d", &intBuff);
	dscadsettings.load_cal = (BYTE)intBuff;
	dscadsettings.current_channel = 0;

	printf("Enter the scan interval (0 for 5 microsec, 1 for 9 microsec): ");
	scanf("%d", &intBuff);

	if( intBuff )
		dscadsettings.scan_interval = SCAN_INTERVAL_9;
	else
		dscadsettings.scan_interval = SCAN_INTERVAL_5;
	

	if( ( result = dscADSetSettings( dscb, &dscadsettings ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// IV. AD SCAN INITIALIZATION
	//
	//	   Initialize the structure containing the AD scan setting and allocate
	//	   memory for our buffer containing sample values.
	//
	//=========================================================================

	/* PRE-FILLED EXAMPLE
	dscadscan.low_channel = 3;
	dscadscan.high_channel = 6;
	dscadscan.gain = GAIN_1;
	*/

	printf( "\nAD SCAN INITIALIZATION\n" );

	printf( "Enter the lowest channel in the scan range (0-15): " );
	scanf("%d", &intBuff);
	dscadscan.low_channel = (BYTE) intBuff;

	printf( "Enter the highest channel in the scan range (0-15): " );
	scanf("%d", &intBuff);
	dscadscan.high_channel = (BYTE) intBuff;

	dscadscan.gain = dscadsettings.gain;

	// allocate memory for buffer
	samples = (DSCSAMPLE*)malloc( sizeof(DSCSAMPLE) * ( dscadscan.high_channel - dscadscan.low_channel + 1 ) );

	getchar();

	//=========================================================================
	// V. SCANNING AND OUTPUT
	//
	//    Perform the actual sampling and then output the results. To calculate
	//	  the actual input voltages, we must convert the sample code (which
	//	  must be cast to a short to get the correct code) and then plug it
	//	  into one of the formulas located in the manual for your board (under
	//	  "A/D Conversion Formulas").
	//=========================================================================

	printf( "\nSCANNING AND OUTPUT\n" );

	do
	{
		if( ( result = dscADScan( dscb, &dscadscan, samples ) ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			fprintf( stderr, "dscADScan error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			free( samples ); // remember to deallocate malloc() memory
			return 0;
		}

		printf( "\nSample readouts:" );

		for( i = 0; i < (dscadscan.high_channel - dscadscan.low_channel)+ 1; i++)
			printf( " %hd", dscadscan.sample_values[i] );

		printf( "\nActual voltages:" );

		for( i = 0; i < (dscadscan.high_channel - dscadscan.low_channel)+ 1; i++)
		{
			if( dscADCodeToVoltage(dscb, dscadsettings, dscadscan.sample_values[i], &voltage) != DE_NONE)
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscADCodeToVoltage error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				free(samples);
				return 0;
			}

			printf( " %5.3lfV", voltage);
		}

		printf( "\n" );

		printf("Enter 'q' to continue or any key to exit\n");
	}
	while(getchar() != 'q');

	//=========================================================================
	// VI. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the memory allocated for sample values
	//	   2. free the driver resources
	//=========================================================================

	free( samples );
	
	dscFree();

	printf( "\nDSCADScan completed.\n" );

	return 0;
} // end main()



#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
