//=============================================================================
// (c) Copyright 2005 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCUserInt.c   v5.9
// Desc: Sample program that demonstrates how to use the User Interrupt functions
// Created by 	KL
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif
#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "../../../../current/dev/source/dscud.h"
#endif

// Linux and QNX
#if defined(linux) || defined(__QNXNTO__) || defined(_WRS_VXWORKS_5_X)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"


#ifdef _WRS_VXWORKS_5_X
#include <selectLib.h>
#define main DMM48DSCUserInt
#else
#include <sys/time.h>
#endif

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif


// var declarations
BYTE result; // returned error code
DSCB dscb;   // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
DSCUSERINT dscuserint;     // structure for setting up user interrupt instead mode
DSCOPTOSTATE dscoptostate; //structure for setting up opto
BYTE config_byte;          // DIO configuration byte
ERRPARAMS errorParams;     // structure for returning error code and error string
BYTE byteBuff;   // temp variable of size byte
int intBuff;     // temp variable of size int
float floatBuff; // temp variables of size float
int port;        // DIO port to output to
int i;
int counter;	 // the global variable to be modified by the user defined function

void count(void * param);  // user function

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. User interrupt setup
//	    IV. Trigger the interrupts (just for demo purposes)
//	     V. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//	  Initialize the driver, using the driver version for validation
	//
	//=========================================================================

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the DMM-48-AT board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf( "\nDMM-48-AT BOARD INITIALIZATION:\n" );

	printf("Enter the base address in hex: ");
	scanf( "%hx", &dsccb.base_address );

	printf("Enter the interrupt : ");
	scanf("%d", &intBuff);
	dsccb.int_level = (BYTE) intBuff;

	if(dscInitBoard(DSC_DMM48, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// III. USER INT INITIALIZATION and Run
	//
	//		Sets up the parameters for the user interrupt. 
	//      User Interrupts for DMM-48-AT has two different interrupt sources:
	//		Opto edge trigger or DIO state change trigger
	//
	//		Opto edge triggers an interrupt upon edge change. 
	//		DIO triggers an interrupt upon any change to any of the DIO ports
	//=========================================================================


	/////////////////////////////
	//  User Interrupt Source  //
	/////////////////////////////

	//	intsource	counter		clksource		description
	//	0			0			0				Counter 1 internal counter
	//	1			0			0				DIO Interruts. Trigger on DIO state change
	//	2			0			0				Opto Interrupts. Trigger on edge
	

	counter =0;		// set counter to 0

	printf("Enter interrupt source (Counter Int 0, DIO Int 1, Opto Int 2): ");
	scanf("%d", &intBuff);
	dscuserint.intsource = (BYTE) intBuff;
	dscuserint.counter = 0;
	if( dscuserint.intsource == 0)
	{
		printf("Enter rate of interrupt: ");
		scanf("%f", &floatBuff);
		dscuserint.rate = (FLOAT)floatBuff;
	}

	if( dscuserint.intsource == 1)
	{
		printf("Enter the DIO configuration byte: ");
		scanf("%d", &intBuff);
		config_byte = (BYTE) intBuff;
		dscDIOSetConfig(dscb, &config_byte);
	}
	else if( dscuserint.intsource == 2)
	{
		for ( i=0; i< 4; i++)
		{
			printf("Enable edge detec (0 or 1) for port %d: ", i);
			scanf("%d", &intBuff);
			dscoptostate.edge_detect_enab[i] = (BYTE) intBuff; 
			if( intBuff == 1)
			{
				printf("Enter edge polarity (0 or 1) for port %d: ", i);
				scanf("%d", &intBuff);
				dscoptostate.edge_polarity[i] = (BYTE) intBuff;
			}
		}
		
		if( dscOptoSetState(dscb, &dscoptostate) != DE_NONE)
		{
			dscGetLastError(&errorParams);
			fprintf( stderr, "dscOptoSetState error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}
	}
	
	if( dscUserInt(dscb, &dscuserint, count) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscUserInt error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//===========================================================================
	//	V. CASES FOR USERINT 
	//
	//      Tests userinterrupt. For DIO users must change the DIO state/value.
	//		For Opto users must change the opto input
	//
	//===========================================================================

	while( counter < 10 )
	{
		/*/////////////////////////////////////
		 The following code is for userint DIO 
		/////////////////////////////////////*/
		if( dscuserint.intsource == 1)
		{
			printf("Enter output port: ");
			scanf("%d", &port);
			printf("Enter output value: ");
			scanf("%d", &intBuff);
			dscDIOOutputByte(dscb, (BYTE)port, (BYTE)intBuff);

			printf("Enter input port: ");
			scanf("%d", &port);
			dscDIOInputByte(dscb, (BYTE)port, &byteBuff);

			printf("Input Byte %d\n", byteBuff);
		}

		/*///////////////////////////////////////
		 The following code is for userint Opto
		///////////////////////////////////////*/
		if( dscuserint.intsource == 2)
		{
			printf("Opto State %x\n", (int) (dscoptostate.dmm48at_oint_state) );
		}

		printf("UserInt counter %d   \r", counter);
		dscSleep(500);
	}

	// cancel interrupts manually for recycled mode or if interrupts are still running
	if( (result = dscCancelOp(dscb)) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscCancelOp error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	if( (result = dscClearUserInterruptFunction(dscb)) != DE_NONE)	// Clear userint settings
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscClearUserInterruptFunction error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;	
	}

	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf( "\nDSCUserInt completed.\n" );

	return 0;
} // end main()



//=============================================================================
//	The user interrupt routine
//
//	Note: If using DOS, you cannot execute dsc driver functions to access the board
//	both in the main program and in the user interrupt routine. This is because
//	the driver locks the board in the main program when it is being used so that
//	no other program can access it and interfere with a function while it is executing.
//	To access the board inside the interrupt routine in DOS, use direct I/O functions
//	( inp(addr), outp(addr,data) ).
//
//=============================================================================

void count(void * param)
{
	counter ++;

	/*
	//////////////////////////////////////
	The Following code is for userint opto
	//////////////////////////////////////
	dscOptoGetState(dsccb, &dscoptostate);
	*/
}

#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
